<?php

namespace Drupal\Tests\marketing_cloud_address\Functional;

use Drupal\Tests\BrowserTestBase;
use League\JsonGuard;
use League\JsonReference;

/**
 * Tests the base marketing_cloud_address module
 *
 * @group marketing_cloud
 * @runTestsInSeparateProcesses
 * @preserveGlobalState disabled
 */
class MarketingCloudAddressTest extends BrowserTestBase {

  /**
   * Modules to install.
   *
   * @var array
   */
  public static $modules = ['marketing_cloud', 'marketing_cloud_address'];

  protected $adminUser;
  protected $service;
  protected $metaSchema;
  protected $moduleConfig;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    // Create user.
    $this->adminUser = $this->drupalCreateUser(['administer_marketing_cloud']);
    $this->drupalLogin($this->adminUser);
    // Set module config
    $this->config('marketing_cloud.settings')
      ->set('client_id', 'testingid')
      ->set('client_secret', 'testingsecret')
      ->set('validate_json', true)
      ->set('validate_schema', true)
      ->set('do_not_send', true)
      ->save();
    // Get schema validator
    $this->metaSchema = JsonReference\Dereferencer::draft6()->dereference('http://json-schema.org/draft-06/schema#');
    //Create service
    $this->service = \Drupal::service('marketing_cloud_address.service');
    // Get marketing_cloud_assets config object
    $this->moduleConfig = \Drupal::config('marketing_cloud_address.settings');
  }

  /**
   * Tests the services and schemas for marketing_cloud_address.
   */
  public function testDefinitions() {
    // Test schema
    $schema = $this->moduleConfig->get("definitions.validate_email.schema");
    $this->assertNotEmpty($schema,"json schema for validate_email is empty.");
    $schema = json_decode($schema);
    $this->assertNotEmpty($schema,"json schema for validate_email is invalid json.");
    $validator = new JsonGuard\Validator($schema, $this->metaSchema);
    $this->assertNotFalse($validator->passes(), "schema validation failed for validate_email.");
    // Test validateEmail against expected inputs
    $data = $this->validateEmailData();
    $result = $this->service
      ->validateEmail($data);
    $this->assertNotFalse($result, 'Valid json failed against the schema in validate_email()');
    $this->assertEquals(
      [
        'url' => 'https://www.exacttargetapis.com/address/v1/validateEmail',
        'data' => json_encode($data)
      ],
      $result
    );
    // Test validateEmail against invalid extra index
    $data['foo'] = 'bar';
    $result = $this->service
      ->validateEmail($data);
    $this->assertFalse($result, 'Failed to detect invalid json against the schema in validate_email()');
    // Test validateEmail against invalid type
    $data['validators'] = 'foobar';
    $result = $this->service
      ->validateEmail($data);
    $this->assertFalse($result, 'Failed to detect invalid json against the schema in validate_email()');
  }

  /**
   * JSON payload for the valid.ateEmail service call
   * @return array
   */
  private function validateEmailData() {
    return [
      'email' => 'help@example.com',
      'validators' => ['SyntaxValidator', 'MXValidator', 'ListDetectiveValidator']
    ];
  }
}
