<?php
/**
 * @file: contains the definition of MarketingCloudExampleWebformPluginHandler
 */

namespace Drupal\marketing_cloud_example\Plugin\WebformHandler;

use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\Config\Definition\Exception\Exception;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Drupal\marketing_cloud\marketing_cloud_sms;

/**
 * Form submission handler.
 *
 * @WebformHandler(
 *   id = "marketing_cloud_example",
 *   label = @Translation("Marketing Cloud Example"),
 *   category = @Translation("Form Handler"),
 *   description = @Translation("Send an SMS on form submission"),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 *   submission = \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_REQUIRED,
 * )
 */
class MarketingCloudExample extends WebformHandlerBase {
  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'short_code' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['short_code'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Short Code'),
      '#description' => $this->t('Short code  - this is the base message code in SF that we will override.'),
      '#default_value' => $this->configuration['short_code'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    if (!$form_state->getErrors()) {
      $this->configuration['short_code'] = $form_state->getValue('short_code');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state, WebformSubmissionInterface $webform_submission) {
    $response = \Drupal::service('marketing_cloud_push.service')
      ->sendMessageToAll('OEl8ODow', [
        "override" => true,
        "messageText" => "New information available!",
        "title" => "Boost your winnings!",
        "subtitle" => "Check your app to see what else is in store.",
        "mutable-content" => true,
        "sendTime" => "2012-10-31 09 =>00",
        "sound" => "MyFile.caf",
        "badge" => "+1",
        "openDirect" => "OD01",
        "customKeys" => [
          "keyA" => "keyA_value",
          "keyB" => "keyB_value"
        ],
        "customPayload" => [ "customA" => "customA_value" ]
      ]);
    var_dump($response);die();

    $values = $webform_submission->getData();
    $mobileNumbers = !is_array($values['mobile_number']) ? [$values['mobile_number']] : $values['mobile_number'];
    $date = new \DateTime();
    $date->modify('-1 day'); // arbitrary time in the past to send instantly
    $messageId = $this->configuration['short_code'];
    $data = [
      'mobileNumbers' => $mobileNumbers,
      'override' => 'true',
      'messageText' => $values['message_text'],
      'SendTime' => $date->format('Y-m-d H:i'),
    ];
    $response = \Drupal::service('marketing_cloud_sms.service')
      ->postMessageToNumber($messageId, $data);
    if (!$response) {
      $message = t('SMS send failed. Please check the log.');
      drupal_set_message($message, 'info');
    } elseif (isset($response['errors'])) {
      $message = t('Message not sent - errors: %errors', array('%errors' => $response['errors']));
      drupal_set_message($message, 'error');
      \Drupal::logger(__METHOD__)->error($message);
    } elseif (!isset($response['result'])) {
      $message = t('Message not sent - errors: %errors', array('%errors' => print_r($response, true)));
      drupal_set_message($message, 'error');
      \Drupal::logger(__METHOD__)->error($message);
    } else {
      $message = t('Response from SalesForce: "%result"', array('%result' => $response['result']));
      drupal_set_message($message, 'info');
      \Drupal::logger(__METHOD__)->info($message);
    }
  }
}
