<?php
/**
 * @file
 * Contains \Drupal\marketing_cloud_messages\MessagesService.
 */

namespace Drupal\marketing_cloud_messages;

use Drupal\marketing_cloud\MarketingCloudService;

/**
 * Class MessagesService
 *
 * For all of the API service calls, a correct JSON data payload is expected.
 * This is then validated against the JSON Schema. This approach minimises
 * any short-term issues with changes in the SF API, provides a sanitized
 * interface to send API calls and leaves flexibility for any modules that
 * want to use this as a base-module.
 *
 * @package Drupal\marketing_cloud
 */
class MessagesService extends MarketingCloudService {

  private $moduleName = 'marketing_cloud_messages';

  /**
   * MessagesService constructor.
   */
  public function __construct() {
  }

  /**
   * Sends an email to a contact.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/messageDefinitionSends.htm
   *
   * @param $triggeredSendDefinitionId
   * @param $json
   * @return bool|null
   */
  public function sendEmail($triggeredSendDefinitionId, $json=NULL) {
    $machineName = 'send_email';
    $json = empty($json) ? new \stdClass() : $json;
    return $this->apiCall($this->moduleName, $machineName, $json, ['[triggeredSendDefinitionId]' => $triggeredSendDefinitionId]);
  }

  /**
   * Gets an email delivery status.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/messageDefinitionSendsDeliveryRecords.htm
   *
   * @param $key
   * @param $recipientSendId
   * @return bool|null
   */
  public function getEmailDeliveryStatus($key, $recipientSendId) {
    $machineName = 'get_email_delivery_status';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[key]' => $key, '[recipientSendId]' => $recipientSendId]);
  }
}
