<?php
/**
 * @file
 * Contains \Drupal\marketing_cloud_platform\PlatformService.
 */

namespace Drupal\marketing_cloud_platform;

use Drupal\marketing_cloud\MarketingCloudService;

/**
 * Class PlatformService
 *
 * For all of the API service calls, a correct JSON data payload is expected.
 * This is then validated against the JSON Schema. This approach minimises
 * any short-term issues with changes in the SF API, provides a sanitized
 * interface to send API calls and leaves flexibility for any modules that
 * want to use this as a base-module.
 *
 * @package Drupal\marketing_cloud
 */
class PlatformService extends MarketingCloudService {

  private $moduleName = 'marketing_cloud_platform';

  /**
   * PlatformService constructor.
   */
  public function __construct() {
  }

  /**
   * Gets endpoint data.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getendpoints.htm
   *
   * @return bool|null
   */
  public function getEndpoints() {
    $machineName = 'get_endpoints';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass());
  }

  /**
   * Returns information about the authenticated token.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/gettokencontext.htm
   *
   * @return bool|null
   */
  public function getTokenContext() {
    $machineName = 'get_token_context';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass());
  }

  /**
   * Gets endpoint data.
   * @see https://developer.salesforce.com/docs/atlas.en-us.noversion.mc-apis.meta/mc-apis/getendpoint.htm
   *
   * @param $endpointType
   * @return bool|null
   */
  public function getEndpoint($endpointType) {
    $machineName = 'get_endpoint';
    return $this->apiCall($this->moduleName, $machineName, new \stdClass(), ['[endpointType]' => $endpointType]);
  }
}
