<?php
/**
 * @file Contains Drupal\marketing_cloud\MarketingCloudSession.
 */

namespace Drupal\marketing_cloud;

/**
 * Class MarketingCloudSession
 *
 * @package Drupal\marketing_cloud
 */
class MarketingCloudSession {

  /**
   * MarketingCloudSession constructor.
   */
  public function __construct() {
  }

  /**
   * Reset the stored token.
   */
  public function resetToken() {
    $config = \Drupal::configFactory()
      ->getEditable('marketing_cloud.settings');
    $config
      ->set('token', FALSE)
      ->set('requesting_token', FALSE)
      ->save();
  }

  /**
   * Create and store token for session.
   *
   * @param bool $forceLogin
   * @return bool|null
   */
  public function token($forceLogin=FALSE) {
    $config = \Drupal::configFactory()->getEditable('marketing_cloud.settings');
    if ($config->get('requesting_token') == TRUE) {
      // Wait n seconds to prevent overloading token request with simultaneous requests.
      sleep($config->get('request_token_wait'));
    }

    $token = $config->get('token');
    if ($token != FALSE && !$forceLogin) {
      return $token;
    }

    $config->set('requesting_token', TRUE)->save();
    $token_requisites = TRUE;

    $clientId = $config->get('client_id');
    if (empty($clientId)) {
      \Drupal::logger(__METHOD__)->error('Bad config data: %missingData', array('%missingData' => 'client_id'));
      $token_requisites = FALSE;
    }
    $clientSecret = $config->get('client_secret');
    if (empty($clientSecret)) {
      \Drupal::logger(__METHOD__)->error('Bad config data: %missingData', array('%missingData' => 'client_secret'));
      $token_requisites = FALSE;
    }
    $url = $config->get('request_token_url');
    if (empty($url)) {
      \Drupal::logger(__METHOD__)->error('Bad config data: %missingData', array('%missingData' => 'request_token_url'));
      $token_requisites = FALSE;
    }

    if (!$config->get('do_not_send')) {
      if ($token_requisites) {
        $loginAttempts = 0;
        while (!$token && $loginAttempts++ < $config->get('login_attempts_max')) {
          $token = $this->_requestToken($url, $clientId, $clientSecret);
        }
      } else {
        $token = FALSE;
      }
    } else {
      $token = TRUE;
    }
    $config->set('token', $token)->save();
    $config->set('requesting_token', ($token === FALSE))->save();

    return $token;
  }

  /**
   * Perform the API call to request a valid token.
   * @param $url
   * @param $clientId
   * @param $clientSecret
   * @return bool
   */
  private function _requestToken($url, $clientId, $clientSecret) {
    try {
      \Drupal::logger(__METHOD__)->info('%message', array('%message' => 'Fetching a new token.'));
      $response = \Drupal::httpClient()->post($url,  [
        'verify' => FALSE,
        'headers' => ['Content-Type' => 'application/x-www-form-urlencoded'],
        'form_params' => [
          'clientId' => $clientId,
          'clientSecret' => $clientSecret],
      ]);
      $data = json_decode($response->getBody());
      return $data->accessToken;
    } catch (RequestException $e) {
      \Drupal::logger(__METHOD__)->error('Request exception, failed to fetch token: %error', array('%error' => $e->getMessage()));
      return FALSE;
    } catch (Exception $e) {
      \Drupal::logger(__METHOD__)->error('Generic exception, failed to fetch token: %error', array('%error' => $e->getMessage()));
      return FALSE;
    }
  }
}
